# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import pytest

import log


@pytest.mark.parametrize(
    'name, levelname',
    [
        ('c', 'CRITICAL'),
        ('critical', 'CRITICAL'),
        ('d', 'DEBUG'),
        ('debug', 'DEBUG'),
        ('e', 'ERROR'),
        ('error', 'ERROR'),
        ('exc', 'ERROR'),
        ('exception', 'ERROR'),
        ('i', 'INFO'),
        ('info', 'INFO'),
        ('w', 'WARNING'),
        ('warn', 'WARNING'),
        ('warning', 'WARNING'),
    ],
)
def test_level_mapping(expect, caplog, name, levelname):
    getattr(log, name)("message")
    expect(caplog.records[-1].levelname) == levelname



"""Implements the "magic" to create `logging` records for the caller."""

import inspect
import logging
from typing import Dict, Tuple

from . import state


def create_logger_record(
    level, message, *args, name: str = '', exc_info=None, **kwargs
) -> bool:
    frame = inspect.currentframe().f_back.f_back.f_back  # type: ignore

    name, parent_name = parse_name(name, frame.f_globals)
    if parent_name in state.silenced:
        return False

    logger = get_logger(name, parent_name)
    if not logger.isEnabledFor(level):
        return False

    record = logger.makeRecord(
        name,
        level,
        fn=frame.f_globals['__file__'],
        lno=frame.f_lineno,
        msg=message,
        args=args,
        exc_info=exc_info,
        extra=kwargs,
        sinfo=None,
    )
    logger.handle(record)
    return True


def parse_name(custom_name: str, frame_info: Dict) -> Tuple[str, str]:
    module_name = custom_name or frame_info['__name__']
    if module_name == '__main__':
        try:
            module_name = frame_info['__file__'].split('.')[0].replace('/', '.')
        except KeyError:
            module_name = 'interactive'
    parent_module_name = module_name.split('.')[0]
    return module_name, parent_module_name


def get_logger(name: str, parent_name: str):
    logger = logging.getLogger(name)
    if not logger.level:
        parent_logger = logging.getLogger(parent_name)
        logger.level = parent_logger.level
    if not logger.level:
        logger.level = logging.root.level
    return logger



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import logging

from log.utils import create_logger_record, parse_name


def describe_create_logger_record():
    def it_uses_the_default_log_level_for_new_loggers(expect, monkeypatch):
        monkeypatch.setattr(logging.root, 'level', logging.INFO)

        expect(create_logger_record(logging.DEBUG, 'hello', name='new')) == False
        expect(create_logger_record(logging.INFO, 'hello', name='new')) == True
        expect(create_logger_record(logging.WARNING, 'hello', name='new')) == True

    def it_inherits_the_parent_logging_level(expect):
        logger = logging.getLogger('root')
        logger.level = logging.WARNING

        expect(create_logger_record(logging.DEBUG, 'hello', name='root.new')) == False
        expect(create_logger_record(logging.INFO, 'hello', name='root.new')) == False
        expect(create_logger_record(logging.WARNING, 'hello', name='root.new')) == True


def describe_parse_name():
    def it_uses_the_filename_when_main(expect):
        frame_info = {'__file__': 'my_package/my_module.py'}
        expect(parse_name('__main__', frame_info)[0]) == 'my_package.my_module'

    def it_handles_interactive_sessions(expect):
        expect(parse_name('__main__', {})[0]) == 'interactive'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['log', 'log.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'minilog',
    'version': '2.0b1',
    'description': 'Minimalistic wrapper for Python logging.',
    'long_description': '# minilog\n\nA minimalistic logging wrapper for Python.\n\n[![Unix Build Status](https://img.shields.io/travis/jacebrowning/minilog/main.svg?label=unix)](https://travis-ci.org/jacebrowning/minilog)\n[![Windows Build Status](https://img.shields.io/appveyor/ci/jacebrowning/minilog/main.svg?label=windows)](https://ci.appveyor.com/project/jacebrowning/minilog)\n[![Coverage Status](https://img.shields.io/coveralls/jacebrowning/minilog/main.svg)](https://coveralls.io/r/jacebrowning/minilog)\n[![Scrutinizer Code Quality](https://img.shields.io/scrutinizer/g/jacebrowning/minilog.svg)](https://scrutinizer-ci.com/g/jacebrowning/minilog/?branch=main)\n[![PyPI Version](https://img.shields.io/pypi/v/minilog.svg)](https://pypi.org/project/minilog)\n[![PyPI License](https://img.shields.io/pypi/l/minilog.svg)](https://pypi.org/project/minilog)\n\n## Usage\n\nEvery project should utilize logging, but for simple use cases, this requires a bit too much boilerplate. Instead of including all of this in your modules:\n\n```python\nimport logging\n\nlog = logging.getLogger(__name__)\n\ndef greet(name):\n    log.info("Hello, %s!", name)\n\nif __name__ == "__main__":\n    logging.basicConfig(\n        level=logging.INFO,\n        format="%(levelname)s: %(name)s: %(message)s",\n    )\n```\n\nwith this package you can simply:\n\n```python\nimport log\n\ndef greet(name):\n    log.info("Hello, %s!", name)\n\nif __name__ == "__main__":\n    log.init()\n```\n\nIt will produce the exact same standard library `logging` records behind the scenes.\n\n## Installation\n\nInstall this library directly into an activated virtual environment:\n\n```text\n$ pip install minilog\n```\n\nor add it to your [Poetry](https://poetry.eustace.io/) project:\n\n```text\n$ poetry add minilog\n```\n\n## Documentation\n\nTo view additional options, please consult the [full documentation](https://minilog.readthedocs.io/en/latest/logging/).\n',
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/minilog',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
